from __future__ import print_function, division

import thorpy
from thorpy import basegrid

class Board(thorpy.BaseGrid):

    def __init__(self, n, nx, ny):
        thorpy.BaseGrid.__init__(self, nx, ny, 0)
        self.n = n
        self.free_cells = [0] * self.nx
        self.v = 1
        self.dh = basegrid.DiagonalHelper(self)
        self.history = []
        self.won = None

    def put(self, column):
        if self.free_cells[column] == self.ny:
            return False
        self.history.append(column)
        self[column, self.free_cells[column]] = self.v
        v = self.check(column, self.free_cells[column])
        self.free_cells[column] += 1
        if v:
            self.won = v
        self.v *= -1
        return True

    def unput(self, k):
        for i in range(k):
            column = self.history.pop()
            self.free_cells[column] -= 1
            self[column, self.free_cells[column]] = 0
        self.won = None
        if k%2 != 0:
            self.v *= -1

    def _check_generator(self, generator, arg_list): #optim: hardcoder et pas passer par check generator
        """Return 0 if no win, else return value of the player."""
        player_value = 0
        counter = 0
        for v in generator(*arg_list):
            if v != player_value:
                counter = 0
                player_value = v
            counter += v
            if abs(counter) == self.n:
                return v
        return 0

    def print(self, map_dict=None):
        if map_dict is None:
            map_dict = {0:"#", 1:"x", -1:"o"}
        for line in range(self.ny-1, -1, -1):
            for v in self.iterline(line):
                rep = map_dict.get(v,v)
                print(rep," ",end="")
            print("")
        print("")

    def check(self, x, y): #optim : pas tester colonne si jeton du dessous est adverse, pas tester diag si plus courte que n, etc...
        #check line
        v =  self._check_generator(self.iterline, [y])
        if v: return v
        #check column
        v = self._check_generator(self.itercolumn, [x])
        if v: return v
        #check diag up
        v = self._check_generator(self.dh.iterdiag_up, [x,y])
        if v: return v
        #check diag down
        v = self._check_generator(self.dh.iterdiag_down, [x,y])
        if v: return v
        return 0


##    def pd(self, column): #debug put, print and print result of put
##        v = self.put(column)
##        self.print()
##        print("\nv", v)

##b = Board(3,4,3)
####b[0,2] = 1
####b[1,1] = 1
####b[2,0] = 1
##
##b[0,0] = 1
##b[1,1] = 1
##b[2,2] = 1
##print("v=",b.check(2,2))
##b.print()



